# -*- coding: utf-8 -*-
from tikit.tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tikit.tencentcloud.tione.v20211111 import models


def parse_cos_str(cos_str):
    """ 解析cos 字符串成结构体

    :param cos_str: 格式如： <bucket>/<cos path>
    :type cos_str: str
    :return:
    :rtype: :class:`tikit.tencentcloud.tione.v20211111.models.CosPathInfo`
    """
    try:
        cos_info = models.CosPathInfo()
        bucket_index = cos_str.index('/')
        cos_info.Bucket = cos_str[:bucket_index]
        if len(cos_info.Bucket) == 0:
            raise TencentCloudSDKException(message='bucket cannot be empty')
        cos_info.Paths = [cos_str[bucket_index + 1:]]
        return cos_info
    except Exception as err:
        raise TencentCloudSDKException(message='cos string is not invalid, should be "<bucket>/<cos path>"')



# -*- coding: utf8 -*-
# Copyright (c) 2017-2021 THL A29 Limited, a Tencent company. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings

from tikit.tencentcloud.common.abstract_model import AbstractModel


class CdbInfo(AbstractModel):
    """出参

    """

    def __init__(self):
        r"""
        :param InstanceName: 数据库实例
注意：此字段可能返回 null，表示取不到有效值。
        :type InstanceName: str
        :param Ip: 数据库IP
注意：此字段可能返回 null，表示取不到有效值。
        :type Ip: str
        :param Port: 数据库端口
注意：此字段可能返回 null，表示取不到有效值。
        :type Port: int
        :param MemSize: 数据库内存规格
注意：此字段可能返回 null，表示取不到有效值。
        :type MemSize: int
        :param Volume: 数据库磁盘规格
注意：此字段可能返回 null，表示取不到有效值。
        :type Volume: int
        :param Service: 服务标识
注意：此字段可能返回 null，表示取不到有效值。
        :type Service: str
        :param ExpireTime: 过期时间
注意：此字段可能返回 null，表示取不到有效值。
        :type ExpireTime: str
        :param ApplyTime: 申请时间
注意：此字段可能返回 null，表示取不到有效值。
        :type ApplyTime: str
        :param PayType: 付费类型
注意：此字段可能返回 null，表示取不到有效值。
        :type PayType: int
        :param ExpireFlag: 过期标识
注意：此字段可能返回 null，表示取不到有效值。
        :type ExpireFlag: bool
        :param Status: 数据库状态
注意：此字段可能返回 null，表示取不到有效值。
        :type Status: int
        :param IsAutoRenew: 续费标识
注意：此字段可能返回 null，表示取不到有效值。
        :type IsAutoRenew: int
        :param SerialNo: 数据库字符串
注意：此字段可能返回 null，表示取不到有效值。
        :type SerialNo: str
        :param ZoneId: ZoneId
注意：此字段可能返回 null，表示取不到有效值。
        :type ZoneId: int
        :param RegionId: RegionId
注意：此字段可能返回 null，表示取不到有效值。
        :type RegionId: int
        """
        self.InstanceName = None
        self.Ip = None
        self.Port = None
        self.MemSize = None
        self.Volume = None
        self.Service = None
        self.ExpireTime = None
        self.ApplyTime = None
        self.PayType = None
        self.ExpireFlag = None
        self.Status = None
        self.IsAutoRenew = None
        self.SerialNo = None
        self.ZoneId = None
        self.RegionId = None


    def _deserialize(self, params):
        self.InstanceName = params.get("InstanceName")
        self.Ip = params.get("Ip")
        self.Port = params.get("Port")
        self.MemSize = params.get("MemSize")
        self.Volume = params.get("Volume")
        self.Service = params.get("Service")
        self.ExpireTime = params.get("ExpireTime")
        self.ApplyTime = params.get("ApplyTime")
        self.PayType = params.get("PayType")
        self.ExpireFlag = params.get("ExpireFlag")
        self.Status = params.get("Status")
        self.IsAutoRenew = params.get("IsAutoRenew")
        self.SerialNo = params.get("SerialNo")
        self.ZoneId = params.get("ZoneId")
        self.RegionId = params.get("RegionId")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class DescribeClusterNodesRequest(AbstractModel):
    """DescribeClusterNodes请求参数结构体

    """

    def __init__(self):
        r"""
        :param InstanceId: 集群实例ID,实例ID形如: emr-xxxxxxxx
        :type InstanceId: str
        :param NodeFlag: 节点标识，取值为：
<li>all：表示获取全部类型节点，cdb信息除外。</li>
<li>master：表示获取master节点信息。</li>
<li>core：表示获取core节点信息。</li>
<li>task：表示获取task节点信息。</li>
<li>common：表示获取common节点信息。</li>
<li>router：表示获取router节点信息。</li>
<li>db：表示获取正常状态的cdb信息。</li>
<li>recyle：表示获取回收站隔离中的节点信息，包括cdb信息。</li>
<li>renew：表示获取所有待续费的节点信息，包括cdb信息，自动续费节点不会返回。</li>
注意：现在只支持以上取值，输入其他值会导致错误。
        :type NodeFlag: str
        :param Offset: 页编号，默认值为0，表示第一页。
        :type Offset: int
        :param Limit: 每页返回数量，默认值为100，最大值为100。
        :type Limit: int
        :param HardwareResourceType: 资源类型:支持all/host/pod，默认为all
        :type HardwareResourceType: str
        :param SearchFields: 支持搜索的字段
        :type SearchFields: list of SearchItem
        """
        self.InstanceId = None
        self.NodeFlag = None
        self.Offset = None
        self.Limit = None
        self.HardwareResourceType = None
        self.SearchFields = None


    def _deserialize(self, params):
        self.InstanceId = params.get("InstanceId")
        self.NodeFlag = params.get("NodeFlag")
        self.Offset = params.get("Offset")
        self.Limit = params.get("Limit")
        self.HardwareResourceType = params.get("HardwareResourceType")
        if params.get("SearchFields") is not None:
            self.SearchFields = []
            for item in params.get("SearchFields"):
                obj = SearchItem()
                obj._deserialize(item)
                self.SearchFields.append(obj)
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class DescribeClusterNodesResponse(AbstractModel):
    """DescribeClusterNodes返回参数结构体

    """

    def __init__(self):
        r"""
        :param TotalCnt: 查询到的节点总数
        :type TotalCnt: int
        :param NodeList: 节点详细信息列表
注意：此字段可能返回 null，表示取不到有效值。
        :type NodeList: list of NodeHardwareInfo
        :param TagKeys: 用户所有的标签键列表
注意：此字段可能返回 null，表示取不到有效值。
        :type TagKeys: list of str
        :param HardwareResourceTypeList: 资源类型列表
注意：此字段可能返回 null，表示取不到有效值。
        :type HardwareResourceTypeList: list of str
        :param RequestId: 唯一请求 ID，每次请求都会返回。定位问题时需要提供该次请求的 RequestId。
        :type RequestId: str
        """
        self.TotalCnt = None
        self.NodeList = None
        self.TagKeys = None
        self.HardwareResourceTypeList = None
        self.RequestId = None


    def _deserialize(self, params):
        self.TotalCnt = params.get("TotalCnt")
        if params.get("NodeList") is not None:
            self.NodeList = []
            for item in params.get("NodeList"):
                obj = NodeHardwareInfo()
                obj._deserialize(item)
                self.NodeList.append(obj)
        self.TagKeys = params.get("TagKeys")
        self.HardwareResourceTypeList = params.get("HardwareResourceTypeList")
        self.RequestId = params.get("RequestId")


class DescribeKeyTabFileRequest(AbstractModel):
    """DescribeKeyTabFile请求参数结构体

    """

    def __init__(self):
        r"""
        :param InstanceId: 集群实例ID
        :type InstanceId: str
        :param UserName: 用户名
        :type UserName: str
        """
        self.InstanceId = None
        self.UserName = None


    def _deserialize(self, params):
        self.InstanceId = params.get("InstanceId")
        self.UserName = params.get("UserName")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class DescribeKeyTabFileResponse(AbstractModel):
    """DescribeKeyTabFile返回参数结构体

    """

    def __init__(self):
        r"""
        :param DownLoadUrl: 下载地址
注意：此字段可能返回 null，表示取不到有效值。
        :type DownLoadUrl: str
        :param RequestId: 唯一请求 ID，每次请求都会返回。定位问题时需要提供该次请求的 RequestId。
        :type RequestId: str
        """
        self.DownLoadUrl = None
        self.RequestId = None


    def _deserialize(self, params):
        self.DownLoadUrl = params.get("DownLoadUrl")
        self.RequestId = params.get("RequestId")


class DescribeServiceConfsRequest(AbstractModel):
    """DescribeServiceConfs请求参数结构体

    """

    def __init__(self):
        r"""
        :param InstanceId: 实例ID
        :type InstanceId: str
        :param IpList: IP列表
        :type IpList: list of str
        :param ConfFileName: 文件名
        :type ConfFileName: str
        :param ServiceType: 服务类型
        :type ServiceType: int
        :param ConfGroupId: 配置组ID
        :type ConfGroupId: int
        """
        self.InstanceId = None
        self.IpList = None
        self.ConfFileName = None
        self.ServiceType = None
        self.ConfGroupId = None


    def _deserialize(self, params):
        self.InstanceId = params.get("InstanceId")
        self.IpList = params.get("IpList")
        self.ConfFileName = params.get("ConfFileName")
        self.ServiceType = params.get("ServiceType")
        self.ConfGroupId = params.get("ConfGroupId")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class DescribeServiceConfsResponse(AbstractModel):
    """DescribeServiceConfs返回参数结构体

    """

    def __init__(self):
        r"""
        :param ServiceConfList: 配置列表
注意：此字段可能返回 null，表示取不到有效值。
        :type ServiceConfList: list of ServiceConf
        :param RequestId: 唯一请求 ID，每次请求都会返回。定位问题时需要提供该次请求的 RequestId。
        :type RequestId: str
        """
        self.ServiceConfList = None
        self.RequestId = None


    def _deserialize(self, params):
        if params.get("ServiceConfList") is not None:
            self.ServiceConfList = []
            for item in params.get("ServiceConfList"):
                obj = ServiceConf()
                obj._deserialize(item)
                self.ServiceConfList.append(obj)
        self.RequestId = params.get("RequestId")


class DescribeUserManagerUserListRequest(AbstractModel):
    """DescribeUserManagerUserList请求参数结构体

    """

    def __init__(self):
        r"""
        :param InstanceId: 集群实例ID
        :type InstanceId: str
        :param PageNo: 页码
        :type PageNo: int
        :param PageSize: 分页的大小
        :type PageSize: int
        :param UserManagerFilter: 查询用户列表过滤器
        :type UserManagerFilter: :class:`tikit.tencentcloud.emr.v20190103.models.UserManagerFilter`
        """
        self.InstanceId = None
        self.PageNo = None
        self.PageSize = None
        self.UserManagerFilter = None


    def _deserialize(self, params):
        self.InstanceId = params.get("InstanceId")
        self.PageNo = params.get("PageNo")
        self.PageSize = params.get("PageSize")
        if params.get("UserManagerFilter") is not None:
            self.UserManagerFilter = UserManagerFilter()
            self.UserManagerFilter._deserialize(params.get("UserManagerFilter"))
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class DescribeUserManagerUserListResponse(AbstractModel):
    """DescribeUserManagerUserList返回参数结构体

    """

    def __init__(self):
        r"""
        :param UserManagerUserList: 用户信息列表
注意：此字段可能返回 null，表示取不到有效值。
        :type UserManagerUserList: list of UserManagerUser
        :param TotalCnt: 总数
注意：此字段可能返回 null，表示取不到有效值。
        :type TotalCnt: int
        :param RequestId: 唯一请求 ID，每次请求都会返回。定位问题时需要提供该次请求的 RequestId。
        :type RequestId: str
        """
        self.UserManagerUserList = None
        self.TotalCnt = None
        self.RequestId = None


    def _deserialize(self, params):
        if params.get("UserManagerUserList") is not None:
            self.UserManagerUserList = []
            for item in params.get("UserManagerUserList"):
                obj = UserManagerUser()
                obj._deserialize(item)
                self.UserManagerUserList.append(obj)
        self.TotalCnt = params.get("TotalCnt")
        self.RequestId = params.get("RequestId")


class MultiDiskMC(AbstractModel):
    """多云盘参数

    """

    def __init__(self):
        r"""
        :param Count: 该类型云盘个数
注意：此字段可能返回 null，表示取不到有效值。
        :type Count: int
        :param Type: 磁盘类型
注意：此字段可能返回 null，表示取不到有效值。
        :type Type: int
        :param Volume: 云盘大小
注意：此字段可能返回 null，表示取不到有效值。
        :type Volume: int
        """
        self.Count = None
        self.Type = None
        self.Volume = None


    def _deserialize(self, params):
        self.Count = params.get("Count")
        self.Type = params.get("Type")
        self.Volume = params.get("Volume")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class NodeHardwareInfo(AbstractModel):
    """节点硬件信息

    """

    def __init__(self):
        r"""
        :param AppId: 用户APPID
注意：此字段可能返回 null，表示取不到有效值。
        :type AppId: int
        :param SerialNo: 序列号
注意：此字段可能返回 null，表示取不到有效值。
        :type SerialNo: str
        :param OrderNo: 机器实例ID
注意：此字段可能返回 null，表示取不到有效值。
        :type OrderNo: str
        :param WanIp: master节点绑定外网IP
注意：此字段可能返回 null，表示取不到有效值。
        :type WanIp: str
        :param Flag: 节点类型。0:common节点；1:master节点
；2:core节点；3:task节点
注意：此字段可能返回 null，表示取不到有效值。
        :type Flag: int
        :param Spec: 节点规格
注意：此字段可能返回 null，表示取不到有效值。
        :type Spec: str
        :param CpuNum: 节点核数
注意：此字段可能返回 null，表示取不到有效值。
        :type CpuNum: int
        :param MemSize: 节点内存
注意：此字段可能返回 null，表示取不到有效值。
        :type MemSize: int
        :param MemDesc: 节点内存描述
注意：此字段可能返回 null，表示取不到有效值。
        :type MemDesc: str
        :param RegionId: 节点所在region
注意：此字段可能返回 null，表示取不到有效值。
        :type RegionId: int
        :param ZoneId: 节点所在Zone
注意：此字段可能返回 null，表示取不到有效值。
        :type ZoneId: int
        :param ApplyTime: 申请时间
注意：此字段可能返回 null，表示取不到有效值。
        :type ApplyTime: str
        :param FreeTime: 释放时间
注意：此字段可能返回 null，表示取不到有效值。
        :type FreeTime: str
        :param DiskSize: 硬盘大小
注意：此字段可能返回 null，表示取不到有效值。
        :type DiskSize: str
        :param NameTag: 节点描述
注意：此字段可能返回 null，表示取不到有效值。
        :type NameTag: str
        :param Services: 节点部署服务
注意：此字段可能返回 null，表示取不到有效值。
        :type Services: str
        :param StorageType: 磁盘类型
注意：此字段可能返回 null，表示取不到有效值。
        :type StorageType: int
        :param RootSize: 系统盘大小
注意：此字段可能返回 null，表示取不到有效值。
        :type RootSize: int
        :param ChargeType: 付费类型
注意：此字段可能返回 null，表示取不到有效值。
        :type ChargeType: int
        :param CdbIp: 数据库IP
注意：此字段可能返回 null，表示取不到有效值。
        :type CdbIp: str
        :param CdbPort: 数据库端口
注意：此字段可能返回 null，表示取不到有效值。
        :type CdbPort: int
        :param HwDiskSize: 硬盘容量
注意：此字段可能返回 null，表示取不到有效值。
        :type HwDiskSize: int
        :param HwDiskSizeDesc: 硬盘容量描述
注意：此字段可能返回 null，表示取不到有效值。
        :type HwDiskSizeDesc: str
        :param HwMemSize: 内存容量
注意：此字段可能返回 null，表示取不到有效值。
        :type HwMemSize: int
        :param HwMemSizeDesc: 内存容量描述
注意：此字段可能返回 null，表示取不到有效值。
        :type HwMemSizeDesc: str
        :param ExpireTime: 过期时间
注意：此字段可能返回 null，表示取不到有效值。
        :type ExpireTime: str
        :param EmrResourceId: 节点资源ID
注意：此字段可能返回 null，表示取不到有效值。
        :type EmrResourceId: str
        :param IsAutoRenew: 续费标志
注意：此字段可能返回 null，表示取不到有效值。
        :type IsAutoRenew: int
        :param DeviceClass: 设备标识
注意：此字段可能返回 null，表示取不到有效值。
        :type DeviceClass: str
        :param Mutable: 支持变配
注意：此字段可能返回 null，表示取不到有效值。
        :type Mutable: int
        :param MCMultiDisk: 多云盘
注意：此字段可能返回 null，表示取不到有效值。
        :type MCMultiDisk: list of MultiDiskMC
        :param CdbNodeInfo: 数据库信息
注意：此字段可能返回 null，表示取不到有效值。
        :type CdbNodeInfo: :class:`tikit.tencentcloud.emr.v20190103.models.CdbInfo`
        :param Ip: 内网IP
注意：此字段可能返回 null，表示取不到有效值。
        :type Ip: str
        :param Destroyable: 此节点是否可销毁，1可销毁，0不可销毁
注意：此字段可能返回 null，表示取不到有效值。
        :type Destroyable: int
        :param Tags: 节点绑定的标签
注意：此字段可能返回 null，表示取不到有效值。
        :type Tags: list of Tag
        :param AutoFlag: 是否是自动扩缩容节点，0为普通节点，1为自动扩缩容节点。
注意：此字段可能返回 null，表示取不到有效值。
        :type AutoFlag: int
        :param HardwareResourceType: 资源类型, host/pod
注意：此字段可能返回 null，表示取不到有效值。
        :type HardwareResourceType: str
        :param IsDynamicSpec: 是否浮动规格，1是，0否
注意：此字段可能返回 null，表示取不到有效值。
        :type IsDynamicSpec: int
        :param DynamicPodSpec: 浮动规格值json字符串
注意：此字段可能返回 null，表示取不到有效值。
        :type DynamicPodSpec: str
        :param SupportModifyPayMode: 是否支持变更计费类型 1是，0否
注意：此字段可能返回 null，表示取不到有效值。
        :type SupportModifyPayMode: int
        :param RootStorageType: 系统盘类型
注意：此字段可能返回 null，表示取不到有效值。
        :type RootStorageType: int
        :param Zone: 可用区信息
注意：此字段可能返回 null，表示取不到有效值。
        :type Zone: str
        :param SubnetInfo: 子网
注意：此字段可能返回 null，表示取不到有效值。
        :type SubnetInfo: :class:`tikit.tencentcloud.emr.v20190103.models.SubnetInfo`
        :param Clients: 客户端
注意：此字段可能返回 null，表示取不到有效值。
        :type Clients: str
        """
        self.AppId = None
        self.SerialNo = None
        self.OrderNo = None
        self.WanIp = None
        self.Flag = None
        self.Spec = None
        self.CpuNum = None
        self.MemSize = None
        self.MemDesc = None
        self.RegionId = None
        self.ZoneId = None
        self.ApplyTime = None
        self.FreeTime = None
        self.DiskSize = None
        self.NameTag = None
        self.Services = None
        self.StorageType = None
        self.RootSize = None
        self.ChargeType = None
        self.CdbIp = None
        self.CdbPort = None
        self.HwDiskSize = None
        self.HwDiskSizeDesc = None
        self.HwMemSize = None
        self.HwMemSizeDesc = None
        self.ExpireTime = None
        self.EmrResourceId = None
        self.IsAutoRenew = None
        self.DeviceClass = None
        self.Mutable = None
        self.MCMultiDisk = None
        self.CdbNodeInfo = None
        self.Ip = None
        self.Destroyable = None
        self.Tags = None
        self.AutoFlag = None
        self.HardwareResourceType = None
        self.IsDynamicSpec = None
        self.DynamicPodSpec = None
        self.SupportModifyPayMode = None
        self.RootStorageType = None
        self.Zone = None
        self.SubnetInfo = None
        self.Clients = None


    def _deserialize(self, params):
        self.AppId = params.get("AppId")
        self.SerialNo = params.get("SerialNo")
        self.OrderNo = params.get("OrderNo")
        self.WanIp = params.get("WanIp")
        self.Flag = params.get("Flag")
        self.Spec = params.get("Spec")
        self.CpuNum = params.get("CpuNum")
        self.MemSize = params.get("MemSize")
        self.MemDesc = params.get("MemDesc")
        self.RegionId = params.get("RegionId")
        self.ZoneId = params.get("ZoneId")
        self.ApplyTime = params.get("ApplyTime")
        self.FreeTime = params.get("FreeTime")
        self.DiskSize = params.get("DiskSize")
        self.NameTag = params.get("NameTag")
        self.Services = params.get("Services")
        self.StorageType = params.get("StorageType")
        self.RootSize = params.get("RootSize")
        self.ChargeType = params.get("ChargeType")
        self.CdbIp = params.get("CdbIp")
        self.CdbPort = params.get("CdbPort")
        self.HwDiskSize = params.get("HwDiskSize")
        self.HwDiskSizeDesc = params.get("HwDiskSizeDesc")
        self.HwMemSize = params.get("HwMemSize")
        self.HwMemSizeDesc = params.get("HwMemSizeDesc")
        self.ExpireTime = params.get("ExpireTime")
        self.EmrResourceId = params.get("EmrResourceId")
        self.IsAutoRenew = params.get("IsAutoRenew")
        self.DeviceClass = params.get("DeviceClass")
        self.Mutable = params.get("Mutable")
        if params.get("MCMultiDisk") is not None:
            self.MCMultiDisk = []
            for item in params.get("MCMultiDisk"):
                obj = MultiDiskMC()
                obj._deserialize(item)
                self.MCMultiDisk.append(obj)
        if params.get("CdbNodeInfo") is not None:
            self.CdbNodeInfo = CdbInfo()
            self.CdbNodeInfo._deserialize(params.get("CdbNodeInfo"))
        self.Ip = params.get("Ip")
        self.Destroyable = params.get("Destroyable")
        if params.get("Tags") is not None:
            self.Tags = []
            for item in params.get("Tags"):
                obj = Tag()
                obj._deserialize(item)
                self.Tags.append(obj)
        self.AutoFlag = params.get("AutoFlag")
        self.HardwareResourceType = params.get("HardwareResourceType")
        self.IsDynamicSpec = params.get("IsDynamicSpec")
        self.DynamicPodSpec = params.get("DynamicPodSpec")
        self.SupportModifyPayMode = params.get("SupportModifyPayMode")
        self.RootStorageType = params.get("RootStorageType")
        self.Zone = params.get("Zone")
        if params.get("SubnetInfo") is not None:
            self.SubnetInfo = SubnetInfo()
            self.SubnetInfo._deserialize(params.get("SubnetInfo"))
        self.Clients = params.get("Clients")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class SearchItem(AbstractModel):
    """搜索字段

    """

    def __init__(self):
        r"""
        :param SearchType: 支持搜索的类型
        :type SearchType: str
        :param SearchValue: 支持搜索的值
        :type SearchValue: str
        """
        self.SearchType = None
        self.SearchValue = None


    def _deserialize(self, params):
        self.SearchType = params.get("SearchType")
        self.SearchValue = params.get("SearchValue")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class ServiceConf(AbstractModel):
    """服务配置

    """

    def __init__(self):
        r"""
        :param ServiceType: 组件类型
注意：此字段可能返回 null，表示取不到有效值。
        :type ServiceType: int
        :param SoftName: 组件名称
注意：此字段可能返回 null，表示取不到有效值。
        :type SoftName: str
        :param FileName: 文件名
注意：此字段可能返回 null，表示取不到有效值。
        :type FileName: str
        :param FileConf: 文件属性
注意：此字段可能返回 null，表示取不到有效值。
        :type FileConf: str
        :param KeyConf: 关键字配置
注意：此字段可能返回 null，表示取不到有效值。
        :type KeyConf: str
        :param OriParam: 文件参数
注意：此字段可能返回 null，表示取不到有效值。
        :type OriParam: str
        """
        self.ServiceType = None
        self.SoftName = None
        self.FileName = None
        self.FileConf = None
        self.KeyConf = None
        self.OriParam = None


    def _deserialize(self, params):
        self.ServiceType = params.get("ServiceType")
        self.SoftName = params.get("SoftName")
        self.FileName = params.get("FileName")
        self.FileConf = params.get("FileConf")
        self.KeyConf = params.get("KeyConf")
        self.OriParam = params.get("OriParam")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class SubnetInfo(AbstractModel):
    """子网信息

    """

    def __init__(self):
        r"""
        :param SubnetName: 子网信息（名字）
注意：此字段可能返回 null，表示取不到有效值。
        :type SubnetName: str
        :param SubnetId: 子网信息（ID）
注意：此字段可能返回 null，表示取不到有效值。
        :type SubnetId: str
        """
        self.SubnetName = None
        self.SubnetId = None


    def _deserialize(self, params):
        self.SubnetName = params.get("SubnetName")
        self.SubnetId = params.get("SubnetId")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class Tag(AbstractModel):
    """标签

    """

    def __init__(self):
        r"""
        :param TagKey: 标签键
        :type TagKey: str
        :param TagValue: 标签值
        :type TagValue: str
        """
        self.TagKey = None
        self.TagValue = None


    def _deserialize(self, params):
        self.TagKey = params.get("TagKey")
        self.TagValue = params.get("TagValue")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class UserManagerFilter(AbstractModel):
    """用户管理列表过滤器

    """

    def __init__(self):
        r"""
        :param UserName: 用户名
注意：此字段可能返回 null，表示取不到有效值。
        :type UserName: str
        """
        self.UserName = None


    def _deserialize(self, params):
        self.UserName = params.get("UserName")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


class UserManagerUser(AbstractModel):
    """用户管理列表信息（用户管理）

    """

    def __init__(self):
        r"""
        :param UserName: 用户名
注意：此字段可能返回 null，表示取不到有效值。
        :type UserName: str
        :param UserGroup: 用户组
注意：此字段可能返回 null，表示取不到有效值。
        :type UserGroup: str
        :param PassWord: 用户密码
注意：此字段可能返回 null，表示取不到有效值。
        :type PassWord: str
        :param ReMark: 备注
注意：此字段可能返回 null，表示取不到有效值。
        :type ReMark: str
        :param CAMUserName: CAM关联的用户名
注意：此字段可能返回 null，表示取不到有效值。
        :type CAMUserName: str
        :param CAMUserUin: 对应的账号UIN
注意：此字段可能返回 null，表示取不到有效值。
        :type CAMUserUin: str
        :param UserType: 用户类型，无需填写
注意：此字段可能返回 null，表示取不到有效值。
        :type UserType: str
        :param CreateTime: 创建时间，无需填写
注意：此字段可能返回 null，表示取不到有效值。
        :type CreateTime: str
        :param SupportDownLoadKeyTab: 是否支持下载keytab
注意：此字段可能返回 null，表示取不到有效值。
        :type SupportDownLoadKeyTab: bool
        :param SupportModifyPwd: 是否支持修改密码
注意：此字段可能返回 null，表示取不到有效值。
        :type SupportModifyPwd: bool
        :param SupportDelete: 是否支持删除
注意：此字段可能返回 null，表示取不到有效值。
        :type SupportDelete: bool
        """
        self.UserName = None
        self.UserGroup = None
        self.PassWord = None
        self.ReMark = None
        self.CAMUserName = None
        self.CAMUserUin = None
        self.UserType = None
        self.CreateTime = None
        self.SupportDownLoadKeyTab = None
        self.SupportModifyPwd = None
        self.SupportDelete = None


    def _deserialize(self, params):
        self.UserName = params.get("UserName")
        self.UserGroup = params.get("UserGroup")
        self.PassWord = params.get("PassWord")
        self.ReMark = params.get("ReMark")
        self.CAMUserName = params.get("CAMUserName")
        self.CAMUserUin = params.get("CAMUserUin")
        self.UserType = params.get("UserType")
        self.CreateTime = params.get("CreateTime")
        self.SupportDownLoadKeyTab = params.get("SupportDownLoadKeyTab")
        self.SupportModifyPwd = params.get("SupportModifyPwd")
        self.SupportDelete = params.get("SupportDelete")
        memeber_set = set(params.keys())
        for name, value in vars(self).items():
            if name in memeber_set:
                memeber_set.remove(name)
        if len(memeber_set) > 0:
            warnings.warn("%s fileds are useless." % ",".join(memeber_set))
        


# -*- coding: utf-8 -*-
from functools import wraps

from tikit.client import Client

_default_client = Client()


def use_default_client(func):
    """装饰器，能把模块的函数转成调用_default_client的方法。
    虽然能把公共的内容都抽离出来，但使用装饰器的方式会导致IDE提示不友好，用户看到的注释不直观。

    :param func:    传入的模块的函数
    :type func:     对外的任意函数
    :return:        装饰器
    :rtype:         函数
    """
    @wraps(func)
    def wrapper(*args, **kw):
        _default_client.guarantee_valid()
        getattr(_default_client, func.__name__)(*args, **kw)
    # wrapper.__doc__ = getattr(_default_client, func.__name__).__doc__
    return wrapper


def describe_cos_buckets():
    """列出所有bucket

    :return:    bucket列表
    :rtype:     dict
    返回的结果如：
    {
      "Owner": {
        "ID": "qcs::cam::uin/100011011262:uin/100011011262",
        "DisplayName": "100011011162"
      },
      "Buckets": {
        "Bucket": [
          {
            "Name": "bucket-58565",
            "Location": "ap-beijing-fsi",
            "CreationDate": "2021-07-21T11:06:00Z",
            "BucketType": "cos"
          },
          {
            "Name": "tai-1300158565",
            "Location": "ap-guangzhou",
            "CreationDate": "2021-10-22T11:04:40Z",
            "BucketType": "cos"
          }
        ]
      }
    }
    """
    _default_client.guarantee_valid()
    return _default_client.describe_cos_buckets()


def upload_to_cos(local_path, bucket, cos_path):
    """
    把本地路径下的文件或者目录上传到cos上

    :param local_path:  本地路径
    :type local_path:   str
    :param bucket:      cos上的桶
    :type bucket:       str
    :param cos_path:    cos上的路径
    :type cos_path:     str
    :return:            None. 错误信息通过 raise 给出。
    :rtype:
    """
    _default_client.guarantee_valid()
    return _default_client.upload_to_cos(local_path, bucket, cos_path)


def describe_cos_path(bucket, path):
    """获取cos的目录的信息。目录下的内容最多显示1000个。要显示目录下的文件和文件夹，请以"/"结尾

    :param bucket:          cos的桶
    :type bucket:           str
    :param path:            路径
    :type path:             str
    :return:                目录信息
    :rtype:                 dict
    :param maker:           从marker开始列出条目
    :type maker:            str
    :param max_keys:        设置单次返回最大的数量,最大为1000.
    :type max_keys:         int
    :param encoding_type:   设置返回结果编码方式,只能设置为url.
    :type encoding_type:    str
    """
    _default_client.guarantee_valid()
    return _default_client.describe_cos_path(bucket, path)


def delete_cos_path(bucket, path):
    """删除cos目录。要删除目录下的文件和文件夹，请以"/"结尾，即：不带斜杠的，当成文件来删除；带斜杠的，当成文件夹来删除

    :param bucket:      cos的桶
    :type bucket:       str
    :param delete_path: 要删除的路径
    :type delete_path:  str
    """
    _default_client.guarantee_valid()
    return _default_client.delete_cos_path(bucket, path)


def download_from_cos(bucket, cos_path, local_path):
    """从cos上下载文件或者目录本地。
    注意：本地文件存在会直接覆盖。 cos_path为目录且local_path 为存在的目录的时候，cos_path的文件夹名会作为子目录保留。

    :param bucket:      cos上的桶
    :type bucket:       str
    :param cos_path:    cos上的路径
    :type cos_path:     str
    :param local_path:  本地路径
    :type local_path:   str
    :return:            None. 错误信息通过 raise 给出。
    :rtype:
    """
    _default_client.guarantee_valid()
    return _default_client.download_from_cos(bucket, cos_path, local_path)



# -*- coding: utf-8 -*-
from prettytable import PrettyTable

from tikit.tencentcloud.tione.v20211111 import models


def group_resources_table(resource_groups):
    """

    :param resource_groups:
    :type resource_groups:   :class:`tikit.tencentcloud.tione.v20211111.models.DescribeBillingResourceGroupsResponse`
    :return:
    :rtype:
    """
    table = PrettyTable()
    table.field_names = [
        "资源组id",
        "资源组名称",
        "可用节点",
        "总节点",
        "已用/总资源",
        "标签"
    ]
    for resource in resource_groups.ResourceGroupSet:
        resource_detail = "CPU {}/{} 核\nMEM {}/{} GB\nGPU {}/{} 卡".format(
            resource.UsedResource.Cpu, resource.TotalResource.Cpu,
            resource.UsedResource.Memory, resource.TotalResource.Memory,
            resource.UsedResource.Gpu, resource.TotalResource.Gpu)
        tag_detail = "\n".join(map(lambda x: "%s:%s" % (x.TagKey, x.TagValue), resource.TagSet))
        table.add_row([resource.ResourceGroupId,
                       resource.ResourceGroupName,
                       resource.FreeInstance,
                       resource.TotalInstance,
                       resource_detail,
                       tag_detail])
    return table


def group_resources_table_str(self):
    return group_resources_table(self).get_string()


def group_resources_table_html(self):
    return group_resources_table(self).get_html_string()


models.DescribeBillingResourceGroupsResponse.__repr__ = group_resources_table_str
models.DescribeBillingResourceGroupsResponse._repr_html_ = group_resources_table_html



# -*- coding: utf-8 -*-
from prettytable import PrettyTable

from tikit.tencentcloud.tione.v20211111 import models


def infer_templates_table(infer_templates):
    """

    :param infer_templates:
    :type infer_templates:   :class:`tikit.tencentcloud.tione.v20211111.models.DescribeInferTemplatesResponse`
    :return:
    :rtype:
    """
    table = PrettyTable()
    table.field_names = [
        "算法框架",
        "算法框架版本号",
        "支持的训练框架集合",
        "镜像标识",
        "镜像url"
    ]
    for framework_template in infer_templates.FrameworkTemplates:
        for image in framework_template.InferTemplates:
            table.add_row([
                framework_template.Framework,
                framework_template.FrameworkVersion,
                framework_template.Groups,
                image.InferTemplateId,
                image.InferTemplateImage
            ])
    return table


def infer_templates_table_str(self):
    return infer_templates_table(self).get_string()


def infer_templates_table_html(self):
    return infer_templates_table(self).get_html_string()


def training_model_table(training_models):
    """

    :param training_models:
    :type training_models:   :class:`tikit.tencentcloud.tione.v20211111.models.DescribeTrainingModelsResponse`
    :return:
    :rtype:
    """
    table = PrettyTable()
    table.field_names = [
        "模型ID",
        "名称",
        "标签",
        "创建时间"
    ]
    for model in training_models.TrainingModels:
        tag_detail = "\n".join(map(lambda x: "%s:%s" % (x.TagKey, x.TagValue), model.Tags))
        table.add_row([
            model.TrainingModelId,
            model.TrainingModelName,
            tag_detail,
            model.CreateTime
        ])
    return table


def training_model_table_str(self):
    return training_model_table(self).get_string()


def training_model_table_html(self):
    return training_model_table(self).get_html_string()


def training_model_version_table(training_model_versions):
    """

    :param training_model_versions:
    :type training_model_versions:   :class:`tikit.tencentcloud.tione.v20211111.models.DescribeTrainingModelVersionsResponse`
    :return:
    :rtype:
    """
    table = PrettyTable()
    table.field_names = [
        "版本ID",
        "模型版本",
        "算法框架",
        "运行环境来源",
        "运行环境",
        "模型指标",
        "COS 路径",
        "创建时间"
    ]
    for version in training_model_versions.TrainingModelVersions:
        reasoning_source = "内置"
        reasoning_env = version.ReasoningEnvironment
        if version.ReasoningEnvironmentSource == "custom" or version.ReasoningEnvironmentSource == "CUSTOM":
            reasoning_source = "自定义"
            reasoning_env = version.ReasoningImageInfo.ImageUrl
        table.add_row([
            version.TrainingModelVersionId,
            version.TrainingModelVersion,
            version.AlgorithmFramework,
            reasoning_source,
            reasoning_env,
            version.TrainingModelIndex,
            version.TrainingModelCosPath,
            version.TrainingModelCreateTime
        ])
    return table


def training_model_version_table_str(self):
    return training_model_version_table(self).get_string()


def training_model_version_table_html(self):
    return training_model_version_table(self).get_html_string()


models.DescribeInferTemplatesResponse.__repr__ = infer_templates_table_str
models.DescribeInferTemplatesResponse._repr_html_ = infer_templates_table_html

models.DescribeTrainingModelsResponse.__repr__ = training_model_table_str
models.DescribeTrainingModelsResponse._repr_html_ = training_model_table_html

models.DescribeTrainingModelVersionsResponse.__repr__ = training_model_version_table_str
models.DescribeTrainingModelVersionsResponse._repr_html_ = training_model_version_table_html



# -*- coding: utf-8 -*-
import os
import random
import string
import json
import time
import jwt
import requests
import base64
import xml.etree.ElementTree as ET
import configparser
import re
import shutil

from pyhive import hive
from hdfs import Client as HdfsClient
from tikit.tencentcloud.wedata.v20210820 import wedata_client
from tikit.tencentcloud.wedata.v20210820 import models as wedata_models
from tikit.tencentcloud.emr.v20190103 import emr_client
from tikit.tencentcloud.emr.v20190103 import models as emr_models
from tikit.tencentcloud.common import credential
from tikit.tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from hdfs.ext.kerberos import KerberosClient
from tqdm import tqdm
from tikit.tencentcloud.tione.v20211111 import models

NONE = 0
KERBEROS = 1
LDAP = 2
SIMPLE = 3

def _get_tmp_path():
    """
    返回随机生成的hdfs临时目录
    :return:    临时目录
    :rtype:     str
    """
    folder = "".join(random.sample(string.ascii_letters + string.digits, random.randint(12, 20)))
    return "/tmp/tikit/%s" % folder


def _get_host_port(server):
    """
    把服务地址解析成主机地址和端口
    :param server:  服务地址
    :type server:   str
    :return:        主机地址和端口
    :rtype:         list
    """
    parts = server.split(":")
    if len(parts) != 2:
        raise Exception('invalid hive_server, should like "10.0.0.1:7001"')
    return parts


def upload_to_hive_by_hdfs(local_path, hdfs_url, hive_server, table_name, database="default", auth="CUSTOM",
                           username=None,
                           password=None, overwrite=False, partition=""):
    """把本地文件的数据导入到hive表中，hdfs作为中间存储。
    过程：先把本地文件上传到hdfs上，然后再从hdfs文件导入到hive中。

    :param local_path:      本地文件或者文件夹。文件夹中不能包含子文件夹。
    :type local_path:       str
    :param hdfs_url:        webhdfs的url，如：http://10.0.3.16:4008
    :type hdfs_url:         str
    :param hive_server:     HiveServer2的地址
    :type hive_server:      str
    :param table_name:      Hive表的名称
    :type table_name:       str
    :param database:        数据库名称
    :type database:         str
    :param auth:            认证的方式
    :type auth:             str
    :param username:        数据库认证的用户名
    :type username:         str
    :param password:        数据库认证的密码
    :type password:         str
    :param overwrite:       是否删掉原来的数据
    :type overwrite:        bool
    :param partition:       分区的选择
    :type partition:        str
    :return:
    :rtype:
    """
    hdfs_path = hdfs_tmp_path = _get_tmp_path()
    hdfs_client = HdfsClient(hdfs_url)
    # 上传文件到一个不存在的目录（斜杠结尾）的时候，hdfs库会把目录当成最终文件
    if os.path.isfile(local_path):
        hdfs_path = os.path.join(hdfs_path, os.path.basename(local_path))
    hdfs_client.upload(hdfs_path, local_path)
    try:
        insert_hdfs_into_hive(hdfs_path, hive_server, table_name, database, auth, username,
                              password, overwrite, partition)
    finally:
        hdfs_client.delete(hdfs_tmp_path, True)


def insert_hdfs_into_hive(hdfs_path, hive_server, table_name, database="default", auth="CUSTOM", username=None,
                          password=None, overwrite=False, partition=""):
    """把hdfs的文件数据导入到hive表中。

    :param hdfs_path:       hdfs上的文件或者文件夹。文件夹中不能包含子文件夹。如：/tmp/file.csv
    :type hdfs_path:        str
    :param hdfs_url:        webhdfs的url，如：http://10.0.3.16:4008
    :type hdfs_url:         str
    :param hive_server:     HiveServer2的地址
    :type hive_server:      str
    :param table_name:      Hive表的名称
    :type table_name:       str
    :param database:        数据库名称
    :type database:         str
    :param auth:            认证的方式
    :type auth:             str
    :param username:        数据库认证的用户名
    :type username:         str
    :param password:        数据库认证的密码
    :type password:         str
    :param overwrite:       是否删掉原来的数据
    :type overwrite:        bool
    :param partition:       分区的选择
    :type partition:        str
    :return:
    :rtype:
    """
    hive_host, hive_port = _get_host_port(hive_server)
    with hive.Connection(host=hive_host, port=hive_port, database=database,
                         username=username, password=password, auth=auth) as conn:
        with conn.cursor() as cursor:
            sql = "LOAD DATA INPATH '{}' {} INTO TABLE {} {}".format(
                hdfs_path,
                "OVERWRITE" if overwrite else "",
                table_name,
                "" if partition == "" else "PARTITION (%s)" % partition
            )
            cursor.execute(sql)


# def export_csv_from_hive(local_path, hive_server, table_name="", sql="", database="default", auth="CUSTOM",
#                          username=None, password=None):
#     """把hive表的数据读出，写到本地的csv文件
#
#     :param local_path:      本地文件或者文件夹。文件夹中不能包含子文件夹。
#     :type local_path:       str
#     :param hive_server:     HiveServer2的地址
#     :type hive_server:      str
#     :param table_name:      Hive表的名称。sql设置时忽略此参数
#     :type table_name:       str
#     :param sql:             查数据sql语句。如：select * from t1
#     :type sql:              str
#     :param database:        数据库名称
#     :type database:         str
#     :param auth:            认证的方式
#     :type auth:             str
#     :param username:        数据库认证的用户名
#     :type username:         str
#     :param password:        数据库认证的密码
#     :type password:         str
#     :return:
#     :rtype:
#     """
#     if local_path.endswith("/") or os.path.isdir(local_path):
#         local_path = os.path.join(local_path, "result.csv")
#     if os.path.exists(local_path):
#         raise Exception('"%s" is already existed' % local_path)
#     local_dir = os.path.dirname(local_path)
#     if local_dir != "" and not os.path.exists(local_dir):
#         os.makedirs(local_dir)
#     # 支持table_name、sql两种方式，sql优先
#     if table_name == "" and sql == "":
#         raise Exception('"table_name" and "sql" cannot both be empty')
#     if sql == "":
#         sql = "SELECT * FROM %s" % table_name
#     # 读出表内容，写入到文件
#     hive_host, hive_port = _get_host_port(hive_server)
#     with hive.Connection(host=hive_host, port=hive_port, database=database,
#                          username=username, password=password, auth=auth) as conn:
#         with conn.cursor() as cursor, open(local_path, 'w') as f:
#             writer = csv.writer(f)
#             cursor.execute(sql)
#             # 测试过使用 cursor.fetchmany(10000)的方式，性能更差（1000万条数据，183M）
#             for row in cursor:
#                 writer.writerow(row)
#     return local_path


def export_from_hive_by_hdfs(local_path, hdfs_url, hive_server, table_name="", sql="", database="default",
                             auth="CUSTOM", username=None, password=None,
                             row_format="row format delimited fields terminated by ','"):
    """导出hive表到本地上，hdfs作为中间存储。对于大文件，这种方式比直接从hive表写到本地的效率更高。
    过程：先把hive导出到hdfs上，然后再从hdfs下载文件到本地。

    :param local_path:      本地的目录
    :type local_path:       str
    :param hdfs_url:        webhdfs的url，如：http://10.0.3.16:4008
    :type hdfs_url:         str
    :param hive_server:     HiveServer2的地址
    :type hive_server:      str
    :param table_name:      Hive表的名称。sql设置时忽略此参数
    :type table_name:       str
    :param sql:             查数据sql语句。如：select * from t1
    :type sql:              str
    :param database:        数据库名称
    :type database:         str
    :param auth:            认证的方式
    :type auth:             str
    :param username:        数据库认证的用户名
    :type username:         str
    :param password:        数据库认证的密码
    :type password:         str
    :param row_format:      行的输出格式
    :type row_format:       str
    :return:
    :rtype:
    """
    if os.path.isfile(local_path):
        raise Exception('"local_path" cannot be file')
    tmp_path = _get_tmp_path()
    hdfs_path = os.path.join(tmp_path, "result")
    hdfs_client = HdfsClient(hdfs_url)
    save_hive_to_hdfs(hdfs_path, hive_server, table_name, sql, database, auth, username, password, row_format)
    try:
        local_dir = os.path.dirname(local_path)
        if local_dir != "" and not os.path.exists(local_dir):
            os.makedirs(local_dir)
        return hdfs_client.download(hdfs_path, local_path)
    finally:
        hdfs_client.delete(hdfs_path, True)


def save_hive_to_hdfs(hdfs_path, hive_server, table_name="", sql="", database="default", auth="CUSTOM",
                      username=None, password=None, row_format="row format delimited fields terminated by ','"):
    """把hive表的数据导出到hdfs上

    :param hdfs_path:       目标hdfs的保存路径
    :type hdfs_path:        str
    :param hive_server:     HiveServer2的地址
    :type hive_server:      str
    :param table_name:      Hive表的名称。sql设置时忽略此参数
    :type table_name:       str
    :param sql:             查数据sql语句。如：select * from t1
    :type sql:              str
    :param database:        数据库名称
    :type database:         str
    :param auth:            认证的方式
    :type auth:             str
    :param username:        数据库认证的用户名
    :type username:         str
    :param password:        数据库认证的密码
    :type password:         str
    :param row_format:      行的输出格式
    :type row_format:       str
    :return:
    :rtype:
    """
    if table_name == "" and sql == "":
        raise Exception('"table_name" and "sql" cannot both be empty')
    hive_host, hive_port = _get_host_port(hive_server)
    with hive.Connection(host=hive_host, port=hive_port, database=database,
                         username=username, password=password, auth=auth) as conn:
        with conn.cursor() as cursor:
            # INSERT OVERWRITE [LOCAL] DIRECTORY directory1 SELECT ... FROM ...
            if sql == "":
                execute_sql = "INSERT OVERWRITE DIRECTORY '{}' {} SELECT {} FROM {}".format(
                    hdfs_path,
                    row_format,
                    "*",
                    table_name
                )
                cursor.execute(execute_sql)
            else:
                execute_sql = "INSERT OVERWRITE DIRECTORY '{}' {} {}".format(
                    hdfs_path,
                    row_format,
                    sql
                )
                cursor.execute(execute_sql)
    return hdfs_path


class HiveInitial(object):
    def __init__(self, client):
        self._client = client
        self._keytab = "/tmp/hive.keytab"
        self._hive_site = os.getenv("SPARK_HOME") + "/conf/hive-site.xml"
        self._hdfs_site = os.getenv("SPARK_HOME") + "/conf/hdfs-site.xml"
        self._core_site = os.getenv("SPARK_HOME") + "/conf/core-site.xml"
        self._yarn_site = os.getenv("SPARK_HOME") + "/conf/yarn-site.xml"
        self._krb_conf = "/etc/krb5.conf"
        self._jwt_key = "tbds@2022"
        self._wedata_auth_info = {}
        self._client_info = client.get_client_info()
        self._hive_conn_cache = {}

        for v in [self._hive_site, self._hdfs_site, self._core_site, self._yarn_site]:
            if os.path.exists(v):
                os.remove(v)

    def __del__(self):
        for v in self._hive_conn_cache:
            self._hive_conn_cache[v].close()

    def spark_hive_initial_wedata(self, wedata_id, source_account=None):
        """为spark初始化wedata hive（调用改方法后，就可以使用spark做hive操作）

       :param wedata_id:       wedata数据源id
       :type wedata_id:        int
       :param source_account:     如果hive为系统源，则需要传使用账户uin
       :type source_account:      str
       :rtype:
       """
        datasource_conn_info = self._get_wedata_security_auth_info(wedata_id, source_account)
        if datasource_conn_info["config_file_empty"]:
            raise Exception("Get hive-site.xml failed")
        if datasource_conn_info["auth_type"] == KERBEROS:
            self._init_kerberos_ticket(datasource_conn_info["principal"], datasource_conn_info["keytab"])

    def hive_initial_wedata(self, wedata_id, source_account=None):
        """获取wedata hive连接句柄

       :param wedata_id:       wedata数据源id
       :type wedata_id:        int
       :param source_account:     如果hive为系统源，则需要传使用账户uin
       :type source_account:      str
       :rtype:
       """
        datasource_conn_info = self._get_wedata_security_auth_info(wedata_id, source_account)
        self._wedata_auth_info = datasource_conn_info

        if datasource_conn_info["auth_type"] == KERBEROS:
            self._init_kerberos_ticket(datasource_conn_info["principal"], datasource_conn_info["keytab"])
            conn = hive.connect(host=datasource_conn_info["host"], port=datasource_conn_info["port"], auth='KERBEROS',
                                kerberos_service_name=datasource_conn_info["username"])
        elif datasource_conn_info["auth_type"] == LDAP:
            conn = hive.connect(host=datasource_conn_info["host"], port=datasource_conn_info["port"], auth='LDAP',
                                username=datasource_conn_info["username"], password=datasource_conn_info["passwd"])
        else:
            kwargs = {
                "host": datasource_conn_info["host"],
                "port": datasource_conn_info["port"],
            }
            if datasource_conn_info["username"]:
                kwargs["username"] = datasource_conn_info["username"]
            if datasource_conn_info["passwd"]:
                kwargs["password"] = datasource_conn_info["passwd"]
            if datasource_conn_info["username"] and datasource_conn_info["passwd"]:
                kwargs["auth"] = "CUSTOM"
            conn = hive.connect(**kwargs)
        self._hive_conn_cache[wedata_id] = conn
        return conn

    def spark_hive_initial(self, emr_id, username=None, keytab=None):
        """为spark初始化emr hive（调用改方法后，就可以使用spark做hive操作）

       :param emr_id:       腾讯云emr id
       :type emr_id:        str
       :param username:     如果使用kerberos认证，则需要传入对应用户名
       :type username:      str
       :param keytab:       keytab文件路径。如果使用集群默认账户（如hadoop），则需要提供keytab路径
       :type keytab:        str
       :rtype:
       """
        emr_conn_info = self._get_emr_security_auth_info(emr_id, username, keytab)
        if emr_conn_info["keytab"] != "":
            self._init_kerberos_ticket(username, emr_conn_info["keytab"])

    def hive_initial(self, emr_id, username=None, keytab=None):
        """获取emr hive连接句柄

       :param emr_id:       腾讯云emr id
       :type emr_id:        str
       :param username:     如果使用kerberos认证，则需要传入对应用户名
       :type username:      str
       :param keytab:       keytab文件路径。如果使用集群默认账户（如hadoop），则需要提供keytab路径
       :type keytab:        str
       :rtype:
       """
        emr_conn_info = self._get_emr_security_auth_info(emr_id, username, keytab)
        if emr_conn_info["principal"] != "":
            service_name = (emr_conn_info["principal"].split("@")[0]).split("/")[0]
            self._init_kerberos_ticket(username, emr_conn_info["keytab"])
            return hive.connect(host=emr_conn_info["host"], port=emr_conn_info["port"], auth='KERBEROS',
                                kerberos_service_name=service_name)
        else:
            return hive.connect(host=emr_conn_info["host"], port=emr_conn_info["port"])

    def hive_initial_custom(
            self,
            host=None,
            port=None,
            scheme=None,
            username=None,
            database='default',
            auth=None,
            configuration=None,
            kerberos_service_name=None,
            password=None,
            check_hostname=None,
            ssl_cert=None,
            thrift_transport=None):
        """Connect to HiveServer2

        :param host: What host HiveServer2 runs on
        :param port: What port HiveServer2 runs on. Defaults to 10000.
        :param auth: The value of hive.server2.authentication used by HiveServer2.
            Defaults to ``NONE``.
        :param configuration: A dictionary of Hive settings (functionally same as the `set` command)
        :param kerberos_service_name: Use with auth='KERBEROS' only
        :param password: Use with auth='LDAP' or auth='CUSTOM' only
        :param thrift_transport: A ``TTransportBase`` for custom advanced usage.
            Incompatible with host, port, auth, kerberos_service_name, and password.

        The way to support LDAP and GSSAPI is originated from cloudera/Impyla:
        https://github.com/cloudera/impyla/blob/255b07ed973d47a3395214ed92d35ec0615ebf62
        /impala/_thrift_api.py#L152-L160
        """
        return hive.connect(
            host=host,
            port=port,
            scheme=scheme,
            username=username,
            database=database,
            auth=auth,
            configuration=configuration,
            kerberos_service_name=kerberos_service_name,
            password=password,
            check_hostname=check_hostname,
            ssl_cert=ssl_cert,
            thrift_transport=thrift_transport)

    def upload_to_wedata_hive(self, wedata_id, local_path, table_name, database='default', overwrite=False,
                              partition='', source_account=None):
        """上传文件到wedata hive

        :param wedata_id:       wedata 数据源id
        :type wedata_id:        int
        :param local_path:     本地文件路径
        :type local_path:      str
        :param table_name:     表名
        :type local_path:      str
        :param database:     数据库
        :type database:      str
        :param overwrite:       是否删掉原来的数据
        :type overwrite:        bool
        :param partition:       分区的选择
        :type partition:        str
        :param source_account:     如果hive为系统源，则需要传使用账户uin
        :type source_account:      str
        :rtype:
        """
        conn = self._get_hive_conn_cache(wedata_id, source_account)
        size = os.stat(local_path).st_size

        with conn.cursor() as cursor:
            hdfs_client = self._get_hdfs_client()
            hdfs_path = os.path.join(_get_tmp_path(), "result")

            def progress_callback(name, bytes):
                pbar.update(2 ** 16)
            with tqdm(total=size, unit='B', unit_scale=True) as pbar:
                pbar.set_description('Uploading')
                hdfs_client.upload(hdfs_path, local_path, progress=progress_callback, overwrite=True)

            cursor.execute("USE " + database)
            sql = "LOAD DATA INPATH '{}' {} INTO TABLE {} {}".format(
                hdfs_path,
                "OVERWRITE" if overwrite else "",
                table_name,
                "" if partition == "" else "PARTITION (%s)" % partition
            )
            print("导入hive中...")
            cursor.execute(sql)
            print("导入完成")

    def export_from_wedata_hive(self, wedata_id, local_path, table_name='', database='default', sql='',
                                row_format="row format delimited fields terminated by ','", source_account=None):
        """导出wedata hive数据到本地

        :param wedata_id:       wedata 数据源id
        :type wedata_id:        int
        :param local_path:     本地文件路径
        :type local_path:      str
        :param table_name:     表名
        :type local_path:      str
        :param database:     数据库
        :type database:      str
        :param sql:             查数据sql语句。如：select * from t1
        :type sql:              str
        :param row_format:      行的输出格式
        :type row_format:       str
        :param source_account:     如果hive为系统源，则需要传使用账户uin
        :type source_account:      str
        :rtype:
        """
        conn = self._get_hive_conn_cache(wedata_id, source_account)
        with conn.cursor() as cursor:
            hdfs_client = self._get_hdfs_client()
            hdfs_path = os.path.join(_get_tmp_path(), table_name)

            cursor.execute("USE " + database)
            if sql == "":
                execute_sql = "INSERT OVERWRITE DIRECTORY '{}' {} SELECT {} FROM {}".format(
                    hdfs_path,
                    row_format,
                    "*",
                    table_name
                )
            else:
                execute_sql = "INSERT OVERWRITE DIRECTORY '{}' {} {}".format(
                    hdfs_path,
                    row_format,
                    sql
                )
            print("导出到hdfs中...")
            cursor.execute(execute_sql)
            print("导出到hdfs完成")

            def progress_callback(name, bytes):
                pbar.update(2 ** 16)
            with tqdm(total=hdfs_client.status(hdfs_path)["length"], unit='B', unit_scale=True) as pbar:
                pbar.set_description('Downloading')
                hdfs_client.download(hdfs_path, local_path, progress=progress_callback, overwrite=True)

    def _get_hive_conn_cache(self, wedata_id, source_account):
        # if wedata_id in self._hive_conn_cache:
        #     return self._hive_conn_cache[wedata_id]
        return self.hive_initial_wedata(wedata_id, source_account)

    def _get_hdfs_client(self):
        hdfs_address = []
        open_kerberos = False
        tree = ET.ElementTree(file=self._hdfs_site)
        root = tree.getroot()
        for v in root:
            if v[0].text.startswith("dfs.namenode.http-address"):
                hdfs_address.append("http://" + v[1].text)
            if v[0].text == "dfs.namenode.kerberos.principal":
                open_kerberos = True
                self._init_kerberos_ticket(self._wedata_auth_info["principal"], self._wedata_auth_info["keytab"])

        if not hdfs_address[0]:
            raise Exception("get hdfs address error")

        if open_kerberos:
            client = KerberosClient(hdfs_address[0])
            return client

        client = HdfsClient(hdfs_address[0])
        return client

    def _get_emr_security_auth_info(self, emr_id, username, keytab=None):
        ret = {
            "host": "",
            "port": "",
            "principal": "",
            "keytab": "",
        }

        cred = credential.Credential(self._client_info["secret_id"], self._client_info["secret_key"])
        client = emr_client.EmrClient(cred, self._client_info["region"])

        node_req = emr_models.DescribeClusterNodesRequest()
        node_req.InstanceId = emr_id
        node_req.NodeFlag = "master"
        node_rsp = client.DescribeClusterNodes(node_req)
        if not node_rsp.NodeList:
            raise Exception("Emr NodeList is empty")
        master_host = node_rsp.NodeList[-1].Ip

        conf_req = emr_models.DescribeServiceConfsRequest()
        conf_req.InstanceId = emr_id
        conf_rsp = client.DescribeServiceConfs(conf_req)
        hive_site = hdfs_site = core_site = yarn_site = {}
        for v in conf_rsp.ServiceConfList:
            tmp_param = json.loads(base64.b64decode(v.OriParam).decode('ascii'))

            if v.FileName == "hdfs-site.xml":
                for v1 in tmp_param:
                    hdfs_site[v1["Name"]] = v1["Value"]
                self._save_xml(hdfs_site, self._hdfs_site)
                continue

            if v.FileName == "core-site.xml":
                for v1 in tmp_param:
                    core_site[v1["Name"]] = v1["Value"]
                self._save_xml(core_site, self._core_site)
                continue

            if v.FileName == "yarn-site.xml":
                for v1 in tmp_param:
                    yarn_site[v1["Name"]] = v1["Value"]
                self._save_xml(yarn_site, self._yarn_site)
                continue

            if v.FileName == "hive-site.xml":
                for v1 in tmp_param:
                    hive_site[v1["Name"]] = v1["Value"]
                    if v1["Name"] == "hive.server2.thrift.bind.host":
                        ret["host"] = v1["Value"]
                    if v1["Name"] == "hive.server2.thrift.port":
                        ret["port"] = v1["Value"]
                    if v1["Name"] == "hive.server2.authentication.kerberos.principal":
                        ret["principal"] = v1["Value"]

                if ret["host"] == "emr-default":
                    ret["host"] = master_host
                    hive_site["hive.server2.thrift.bind.host"] = master_host
                    ret["principal"] = ret["principal"].replace("_HOST", master_host)

                self._save_xml(hive_site, self._hive_site)
                if "hive.server2.authentication" in hive_site and hive_site["hive.server2.authentication"] == "kerberos":
                    if not username:
                        raise Exception("The authentication mode is Kerberos, username is required")
                    if (username and username in ["root", "hadoop"]) and not keytab:
                        raise Exception("The %s account must have a KeyTab" % username)
                    if keytab:
                        shutil.copyfile(keytab, self._keytab)
                    else:
                        keytab_req = emr_models.DescribeKeyTabFileRequest()
                        keytab_req.InstanceId = emr_id
                        keytab_req.UserName = username
                        keytab_rsp = client.DescribeKeyTabFile(keytab_req)

                        if not keytab_rsp.DownLoadUrl:
                            raise Exception("DownLoadUrl is empty")
                        download_rsp = requests.get(keytab_rsp.DownLoadUrl)
                        with open(self._keytab, "w") as f:
                            f.write(download_rsp.text)
                    ret["keytab"] = self._keytab

            if v.FileName == "krb5.conf":
                realm = kdc = admin_server = ""
                port = 88
                tmp_param = json.loads(base64.b64decode(v.OriParam).decode('ascii'))
                for v1 in tmp_param:
                    if v1["Name"] == "active_kdc_server_ip":
                        kdc = v1["Value"]
                    if v1["Name"] == "active_admin_server":
                        admin_server = v1["Value"]
                    if v1["Name"] == "realm":
                        realm = v1["Value"]
                    if v1["Name"] == "kdc_server_port":
                        port = v1["Value"]

                realms = """[realms]
    %s = {
        kdc = %s
        admin_server = %s
    }
[domain_realm]""" % (realm.upper(), kdc + ":" + port, admin_server)
                default_realm = "default_realm = " + realm.upper()

                with open(self._krb_conf, "r") as f:
                    krb5_content = f.read()

                with open(self._krb_conf, "r+") as f:
                    a = re.sub("default_realm =.*", default_realm, krb5_content)
                    b = re.sub("\[realms\][\s\S]*\[domain_realm\]", realms, a)
                    f.write(b)

        if not hive_site:
            raise Exception("hive-site.xml does not exist")
        return ret

    def _save_xml(self, data, file):
        self._crete_default_xml(file)
        tree = ET.ElementTree(file=file)
        root = tree.getroot()
        root.clear()
        for k in data:
            prop = ET.SubElement(root, "property")
            name = ET.SubElement(prop, "name")
            name.text = k
            val = ET.SubElement(prop, "value")
            val.text = data[k]
        tree.write(file)

    def _crete_default_xml(self, file):
        if not os.path.exists(file):
            r = ET.Element('configuration')
            t = ET.ElementTree(r)
            t.write(file)

    def _init_kerberos_ticket(self, principal, keytab):
        active_str = 'kinit -kt {0} {1}'.format(keytab, principal)
        if os.system(active_str) != 0:
            raise Exception("kinit error")

    def _get_wedata_security_auth_info(self, wedata_id, source_account):
        # 获取数据源信息
        datasource_rsp = self._call_inside_action("DescribeDatasource", {
            "Id": wedata_id
        })
        if "Data" not in datasource_rsp["Response"] or not datasource_rsp["Response"]["Data"]:
            raise Exception(datasource_rsp)
        if datasource_rsp["Response"]["Data"]["Type"] != "HIVE":
            raise Exception("datasource type is not HIVE")
        datasource = datasource_rsp["Response"]["Data"]
        params = datasource["Params"]

        # 自定义源
        if datasource["Category"] == "DB":
            custom_file_rsp = self._call_inside_action("DescribeDataSourceConfigFiles", {
                "Id": wedata_id
            })
            host, port, config_file_empty = self._save_config_files(custom_file_rsp, datasource["Category"])

            if params["authentication"] == "none":
                return {
                    "host": params["ip"],
                    "port": params["port"],
                    "auth_type": NONE,
                    "keytab": "",
                    "principal": "",
                    "username": "hadoop", #TODO 先默认使用hadoop用户
                    "passwd": "",
                    "config_file_empty": config_file_empty
                }
            elif params["authentication"] == "Kerberos":
                return {
                    "host": host,
                    "port": port,
                    "auth_type": KERBEROS,
                    "keytab": self._keytab,
                    "principal": params["principal"],
                    "username": "hadoop", #TODO 先默认使用hadoop用户
                    "passwd": "",
                    "config_file_empty": config_file_empty
                }
            else:
                raise Exception("Unsupported custom source authentication method")

        # 系统源
        if source_account is None:
            raise Exception("Param source_account cannot be None")
        sys_conf_file_rsp = self._call_inside_action("DescribeConfigFiles", {
            "ClusterId": datasource["ClusterId"]
        })
        host, port, config_file_empty = self._save_config_files(sys_conf_file_rsp, datasource["Category"])

        # 安全认证
        auth_rsp = self._call_inside_action("QueryAccountMapping", {
            "clusterId": datasource["ClusterId"],
            "projectId": datasource["OwnerProjectId"],
            "sourceAccount": source_account,
        })
        if "Data" not in auth_rsp or auth_rsp["Data"] is None:
            raise Exception(auth_rsp)
        auth_data = auth_rsp["Data"]

        if auth_data["authenticationType"] == KERBEROS:  # kerberos
            if auth_data["securityKey"] is None:
                raise Exception("keytab is empty")
            keytab = base64.b64decode(auth_data["securityKey"])
            with open(self._keytab, "w") as f:
                f.write(str(keytab))

        return {
            "host": host,
            "port": port,
            "auth_type": auth_data["authenticationType"],
            "keytab": self._keytab,
            "principal": auth_data["targetAccount"] + "@" + auth_data["clusterId"].upper(),
            "username": auth_data["targetAccount"],
            "passwd": auth_data["securityKey"],
            "config_file_empty": config_file_empty
        }

    def _call_inside_action(self, action, payload):
        req = models.DescribeInsideActionRequest()
        req.ActionParam = action
        req.PayloadParam = base64.b64encode(json.dumps(payload).encode("utf-8")).decode('ascii')
        rsp = self._client_info["tione_client"].DescribeInsideAction(req)
        return json.loads(rsp.Data)

    def _save_config_files(self, config_file_rsp, category):
        if "Data" not in config_file_rsp["Response"]:
            raise Exception(config_file_rsp["Response"])
        config_file = config_file_rsp["Response"]["Data"]

        have_hive_site_file = have_hdfs_site_file = False
        have_krb5_conf_file = have_keytab_file = False
        for v in config_file:
            if v["ConfigFileName"].endswith("hive-site.xml"):
                with open(self._hive_site, "w") as f:
                    f.write(v["ConfigFileDetail"])
                have_hive_site_file = True
            if v["ConfigFileName"].endswith("hdfs-site.xml"):
                with open(self._hdfs_site, "w") as f:
                    f.write(v["ConfigFileDetail"])
                have_hdfs_site_file = True
            if v["ConfigFileName"].endswith("krb5.conf"):
                with open(self._krb_conf, "w") as f:
                    f.write(v["ConfigFileDetail"])
                have_krb5_conf_file = True
            if v["ConfigFileName"].endswith(".keytab"):
                with open(self._keytab, "wb") as f:
                    f.write(base64.b64decode(v["ConfigFileBaseDetail"]))
                have_keytab_file = True

        if not have_hive_site_file:
            raise Exception("Please upload hive-site.xml on the wedata page")
        if not have_hdfs_site_file:
            raise Exception("Please upload hdfs-site.xml on the wedata page")

        host = port = ""
        self._crete_default_xml(self._hive_site)
        tree = ET.ElementTree(file=self._hive_site)
        root = tree.getroot()
        for v in root:
            if v[0].text == "hive.server2.thrift.bind.host":
                host = v[1].text
            if v[0].text == "hive.server2.thrift.port":
                port = v[1].text
            if v[0].text == "hive.server2.authentication" and v[1].text == "kerberos":
                if not have_krb5_conf_file and category == "DB":
                    raise Exception("Please upload krb5.conf on the wedata page")
                if not have_keytab_file and category == "DB":
                    raise Exception("Please upload keytab on the wedata page")

        return host, port, config_file == "" or len(config_file) == 0

    def _get_wedata_jwt_token(self, jwt_payload):
        jwt_headers = {
            "typ": "JWT",
            "alg": "HS256"
        }
        jwt_token = "Bearer " + jwt.encode(payload=jwt_payload, key=self._jwt_key, algorithm='HS256',
                                           headers=jwt_headers)
        return jwt_token



from setuptools import setup, find_packages
import io


def requirements():
    with open('requirements.txt', 'r') as fileobj:
        requirements = [line.strip() for line in fileobj]
        return requirements


def long_description():
    with io.open('README.md', 'r', encoding='utf8') as fileobj:
        return fileobj.read()


setup(
    name='tikit',
    version='1.5.2.230625',
    url='https://cloud.tencent.com/',
    license='MIT',
    author='TI PLATFORM TEAM',
    author_email='TI_Platform@tencent.com',
    description='Kit for TI PLATFORM',
    long_description=long_description(),
    packages=find_packages(),
    package_data={'requirements': ['requirements.txt']},
    install_requires=requirements(),
    entry_points={
        'console_scripts': [
            'tikit = tikit.cli:cli',
        ]
    },
    include_package_data=True
)
